@@ -1,3 +1,10 @@
+##
+# name:      pQuery::DOM
+# abstract:  A DOM Class for pQuery
+# author:    Ingy döt Net <ingy@cpan.org>
+# license:   perl
+# copyright: 2008, 2011
+
 package pQuery::DOM;
 use strict;
 use warnings;
@@ -91,9 +98,9 @@ sub new {
     return $self;
 }
 
-################################################################################
+#------------------------------------------------------------------------------#
 # pQuery::DOM Class Methods
-################################################################################
+#------------------------------------------------------------------------------#
 sub fromHTML {
     my ($class, $html) = @_;
     my $dom;
@@ -122,9 +129,9 @@ sub createComment {
     return $class->fromHTML('<!--' . $comment . '-->');
 }
 
-################################################################################
+#------------------------------------------------------------------------------#
 # DOM Object Methods
-################################################################################
+#------------------------------------------------------------------------------#
 sub toHTML {
     my $self = shift;
 
@@ -159,6 +166,7 @@ sub getElementsByTagName {
     my ($self, $tag) = @_;
     my $found = [];
     _find($self, $found, sub { $_->{_tag} eq $tag or $tag eq "*" });
+    shift @$found if @$found and $found->[0] == $self;
     return $found;
 }
 
@@ -236,6 +244,22 @@ sub lastChild {
     return $_[0]->{_content}[-1];
 }
 
+sub firstChildRef {
+    my $content = $_[0]->{_content} or return;
+    for (my $i = 0; $i < @$content; $i++) {
+        return $content->[$i] if ref $content->[$i];
+    }
+    return;
+}
+
+sub lastChildRef {
+    my $content = $_[0]->{_content} or return;
+    for (my $i = @$content - 1; $i >= 0; $i--) {
+        return $content->[$i] if ref $content->[$i];
+    }
+    return;
+}
+
 sub appendChild {
     my ($self, $elem) = @_;
     return unless defined $elem;
@@ -248,6 +272,26 @@ sub previousSibling {
     die "pQuery::DOM does not support the previousSibling method";
 }
 
+sub nextSiblingRef {
+    my $content = $_[0]->parentNode->{_content} or return;
+    my $found = 0;
+    for (my $i = 0; $i < @$content; $i++) {
+        return $content->[$i] if $found and ref $content->[$i];
+        $found = 1 if ref($content->[$i]) and $content->[$i] == $_[0];
+    }
+    return;
+}
+
+sub previousSiblingRef {
+    my $content = $_[0]->parentNode->{_content} or return;
+    my $found = 0;
+    for (my $i = @$content - 1; $i >= 0; $i--) {
+        return $content->[$i] if $found and ref $content->[$i];
+        $found = 1 if ref($content->[$i]) and $content->[$i] == $_[0];
+    }
+    return;
+}
+
 sub nextSibling {
     die "pQuery::DOM does not support the nextSibling method";
 }
@@ -256,16 +300,16 @@ sub attributes {
     die "pQuery::DOM::attributes not yet implemented";
 }
 
-################################################################################
+#------------------------------------------------------------------------------#
 # Common pQuery method mistakes
-################################################################################
+#------------------------------------------------------------------------------#
 # sub text {
 #     confess "Invalid method 'text' called on pQuery::DOM object";
 # }
 
-################################################################################
+#------------------------------------------------------------------------------#
 # Helper Functions
-################################################################################
+#------------------------------------------------------------------------------#
 sub _to_html {
     my ($elem, $html) = @_;
     if (not ref $elem) {
@@ -292,6 +336,7 @@ sub _to_html {
     }
     $$html .= '</' . $elem->{_tag} . '>';
 }
+# XXX "work around vim hilight bug
 
 sub _find {
     my ($elem, $found, $test) = @_;
@@ -305,10 +350,6 @@ sub _find {
 
 1;
 
-=head1 NAME
-
-pQuery::DOM - A DOM Class for pQuery
-
 =head1 SYNOPSIS
 
     my $dom = pQuery::DOM->fromHTML('<p>I <b>Like</b> Pie</p>';
@@ -328,7 +369,9 @@ HTML::TreeBuilder and HTML::Element. As such, text nodes are just
 strings and therefore cannot have methods called on them.
 
 This implies that the DOM methods previousSibling and nextSibling
-wouldn't really work correctly. Therefore they are not implemented.
+wouldn't really work correctly. Therefore they are not
+implemented. However, previousSiblingRef and nextSiblingRef are
+implemented. See below.
 
 To deal with children, use the childNodes method which returns a list
 of all the child nodes. Then you can use standard Perl idioms to
@@ -460,17 +503,31 @@ Adds a node (or a string) to the end of the current node's children.
 
 =back
 
-=head1 AUTHOR
+=head2 Non-standard Object Methods
 
-Ingy döt Net <ingy@cpan.org>
+These methods are variants of standard methods, but guarantee that the
+result, if found, is another pQuery::DOM node.
 
-=head1 COPYRIGHT
+=over
 
-Copyright (c) 2008. Ingy döt Net.
+=item firstChildRef()
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+Returns the first child node which is actually a pQuery::DOM node and
+not a string.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+=item lastChildRef()
 
-=cut
+Returns the last child node which is actually a pQuery::DOM node and
+not a string.
+
+=item previousSiblingRef()
+
+Returns the previous sibling node which is actually a pQuery::DOM node
+and not a string.
+
+=item nextSiblingRef()
+
+Returns the next sibling node which is actually a pQuery::DOM node and
+not a string.
+
+=back
